@@ -1,3 +1,4 @@
+import collections
 from os.path import dirname, join
 from typing import (
     Any,
@@ -7,6 +8,7 @@ from typing import (
     Iterator,
     List,
     Optional,
+    Set,
     Type,
     Union,
     cast,
@@ -15,7 +17,6 @@ from typing import (
 
 from django.apps import apps
 from django.conf import settings
-from django.contrib import contenttypes
 from django.db.models import (
     AutoField,
     BooleanField,
@@ -26,18 +27,16 @@ from django.db.models import (
     Model,
     OneToOneField,
 )
+from django.db.models.fields import NOT_PROVIDED
 from django.db.models.fields.proxy import OrderWrt
 from django.db.models.fields.related import (
     ReverseManyToOneDescriptor as ForeignRelatedObjectsDescriptor,
 )
-from django.db.models.fields.reverse_related import (
-    ManyToManyRel,
-    ManyToOneRel,
-    OneToOneRel,
-)
+from django.db.models.fields.reverse_related import ManyToOneRel, OneToOneRel
 
 from . import generators, random_gen
 from ._types import M, NewM
+from .content_types import BAKER_CONTENTTYPES
 from .exceptions import (
     AmbiguousModelName,
     CustomBakerNotFound,
@@ -46,8 +45,17 @@ from .exceptions import (
     ModelNotFound,
     RecipeIteratorEmpty,
 )
-from .utils import seq  # NoQA: enable seq to be imported from baker
-from .utils import import_from_str
+from .utils import (
+    import_from_str,
+    seq,  # noqa: F401 - Enable seq to be imported from recipes
+)
+
+if BAKER_CONTENTTYPES:
+    from django.contrib.contenttypes import models as contenttypes_models
+    from django.contrib.contenttypes.fields import GenericForeignKey, GenericRelation
+else:
+    contenttypes_models = None
+    GenericRelation = None
 
 recipes = None
 
@@ -62,6 +70,10 @@ def _valid_quantity(quantity: Optional[Union[str, int]]) -> bool:
     return quantity is not None and (not isinstance(quantity, int) or quantity < 1)
 
 
+def seed(seed: Union[int, float, str, bytes, bytearray, None]) -> None:
+    Baker.seed(seed)
+
+
 @overload
 def make(
     _model: Union[str, Type[M]],
@@ -73,8 +85,7 @@ def make(
     _using: str = "",
     _bulk_create: bool = False,
     **attrs: Any,
-) -> M:
-    ...
+) -> M: ...
 
 
 @overload
@@ -89,8 +100,7 @@ def make(
     _bulk_create: bool = False,
     _fill_optional: Union[List[str], bool] = False,
     **attrs: Any,
-) -> List[M]:
-    ...
+) -> List[M]: ...
 
 
 def make(
@@ -107,7 +117,7 @@ def make(
 ):
     """Create a persisted instance from a given model its associated models.
 
-    It fill the fields with random values or you can specify which
+    Baker fills the fields with random values, or you can specify which
     fields you want to define its values by yourself.
     """
     _save_kwargs = _save_kwargs or {}
@@ -142,8 +152,7 @@ def prepare(
     _save_related: bool = False,
     _using: str = "",
     **attrs,
-) -> M:
-    ...
+) -> M: ...
 
 
 @overload
@@ -154,8 +163,7 @@ def prepare(
     _using: str = "",
     _fill_optional: Union[List[str], bool] = False,
     **attrs,
-) -> List[M]:
-    ...
+) -> List[M]: ...
 
 
 def prepare(
@@ -168,7 +176,7 @@ def prepare(
 ):
     """Create but do not persist an instance from a given model.
 
-    It fill the fields with random values or you can specify which
+    Baker fills the fields with random values, or you can specify which
     fields you want to define its values by yourself.
     """
     attrs.update({"_fill_optional": _fill_optional})
@@ -181,8 +189,8 @@ def prepare(
             baker.prepare(_save_related=_save_related, **attrs)
             for i in range(_quantity)
         ]
-    else:
-        return baker.prepare(_save_related=_save_related, **attrs)
+
+    return baker.prepare(_save_related=_save_related, **attrs)
 
 
 def _recipe(name: str) -> Any:
@@ -235,7 +243,7 @@ class ModelFinder:
             model = None
 
         if not model:
-            raise ModelNotFound("Could not find model '%s'." % name.title())
+            raise ModelNotFound(f"Could not find model '{name.title()}'.")
 
         return model
 
@@ -252,8 +260,8 @@ class ModelFinder:
 
         if name in cast(List, self._ambiguous_models):
             raise AmbiguousModelName(
-                "%s is a model in more than one app. "
-                'Use the form "app.model".' % name.title()
+                f"{name.title()} is a model in more than one app. "
+                'Use the form "app.model".'
             )
 
         return cast(Dict, self._unique_models).get(name)
@@ -280,10 +288,7 @@ class ModelFinder:
 
 
 def is_iterator(value: Any) -> bool:
-    if not hasattr(value, "__iter__"):
-        return False
-
-    return hasattr(value, "__next__")
+    return isinstance(value, collections.abc.Iterator)
 
 
 def _custom_baker_class() -> Optional[Type]:
@@ -301,28 +306,36 @@ def _custom_baker_class() -> Optional[Type]:
     try:
         baker_class = import_from_str(custom_class_string)
 
-        for required_function_name in ["make", "prepare"]:
+        for required_function_name in ("make", "prepare"):
             if not hasattr(baker_class, required_function_name):
                 raise InvalidCustomBaker(
-                    'Custom Baker classes must have a "%s" function'
-                    % required_function_name
+                    f'Custom Baker classes must have a "{required_function_name}" function'
                 )
 
         return baker_class
     except ImportError:
         raise CustomBakerNotFound(
-            "Could not find custom baker class '%s'" % custom_class_string
+            f"Could not find custom baker class '{custom_class_string}'"
         )
 
 
 class Baker(Generic[M]):
+    SENTINEL = object()
+
     attr_mapping: Dict[str, Any] = {}
     type_mapping: Dict = {}
 
+    _global_seed: Union[object, int, float, str, bytes, bytearray, None] = SENTINEL
+
     # Note: we're using one finder for all Baker instances to avoid
     # rebuilding the model cache for every make_* or prepare_* call.
     finder = ModelFinder()
 
+    @classmethod
+    def seed(cls, seed: Union[int, float, str, bytes, bytearray, None]) -> None:
+        random_gen.baker_random.seed(seed)
+        cls._global_seed = seed
+
     @classmethod
     def create(
         cls,
@@ -403,15 +416,12 @@ class Baker(Generic[M]):
         params.update(attrs)
         return self._make(**params)
 
-    def get_fields(self) -> Any:
-        return set(self.model._meta.get_fields()) - set(self.get_related())
+    def get_fields(self) -> Set[Any]:
+        return set(self.model._meta.get_fields()) - set(
+            self.model._meta.related_objects
+        )
 
-    def get_related(
-        self,
-    ) -> List[Union[ManyToOneRel, OneToOneRel, ManyToManyRel]]:
-        return [r for r in self.model._meta.related_objects]
-
-    def _make(
+    def _make(  # noqa: C901
         self,
         commit=True,
         commit_related=True,
@@ -441,7 +451,7 @@ class Baker(Generic[M]):
                         self.m2m_dict[field.name] = self.model_attrs.pop(field.name)
             # is an _id relation that has a sequence defined
             elif (
-                (isinstance(field, OneToOneField) or isinstance(field, ForeignKey))
+                isinstance(field, (OneToOneField, ForeignKey))
                 and hasattr(field, "attname")
                 and field.attname in self.iterator_attrs
             ):
@@ -463,9 +473,7 @@ class Baker(Generic[M]):
                 try:
                     self.model_attrs[field.name] = next(self.iterator_attrs[field.name])
                 except StopIteration:
-                    raise RecipeIteratorEmpty(
-                        "{0} iterator is empty.".format(field.name)
-                    )
+                    raise RecipeIteratorEmpty(f"{field.name} iterator is empty.")
 
         instance = self.instance(
             self.model_attrs,
@@ -474,7 +482,7 @@ class Baker(Generic[M]):
             _save_kwargs=_save_kwargs,
         )
         if commit:
-            for related in self.get_related():
+            for related in self.model._meta.related_objects:
                 self.create_by_related_name(instance, related)
 
         if _refresh_after_create:
@@ -493,17 +501,40 @@ class Baker(Generic[M]):
         self, attrs: Dict[str, Any], _commit, _save_kwargs, _from_manager
     ) -> M:
         one_to_many_keys = {}
+        auto_now_keys = {}
+        generic_foreign_keys = {}
+
         for k in tuple(attrs.keys()):
             field = getattr(self.model, k, None)
+
+            if not field:
+                continue
+
             if isinstance(field, ForeignRelatedObjectsDescriptor):
                 one_to_many_keys[k] = attrs.pop(k)
 
+            if hasattr(field, "field") and (
+                getattr(field.field, "auto_now_add", False)
+                or getattr(field.field, "auto_now", False)
+            ):
+                auto_now_keys[k] = attrs[k]
+
+            if BAKER_CONTENTTYPES and isinstance(field, GenericForeignKey):
+                generic_foreign_keys[k] = {
+                    "value": attrs.pop(k),
+                    "content_type_field": field.ct_field,
+                    "object_id_field": field.fk_field,
+                }
+
         instance = self.model(**attrs)
-        # m2m only works for persisted instances
+
+        self._handle_generic_foreign_keys(instance, generic_foreign_keys)
+
         if _commit:
             instance.save(**_save_kwargs)
             self._handle_one_to_many(instance, one_to_many_keys)
             self._handle_m2m(instance)
+            self._handle_auto_now(instance, auto_now_keys)
 
             if _from_manager:
                 # Fetch the instance using the given Manager, e.g.
@@ -535,24 +566,23 @@ class Baker(Generic[M]):
         # error for non existing fields
         if isinstance(self.fill_in_optional, (tuple, list, set)):
             # parents and relations
-            wrong_fields = set(self.fill_in_optional) - set(
+            wrong_fields = set(self.fill_in_optional) - {
                 f.name for f in self.get_fields()
-            )
+            }
             if wrong_fields:
                 raise AttributeError(
-                    "_fill_optional field(s) %s are not related to model %s"
-                    % (list(wrong_fields), self.model.__name__)
+                    f"_fill_optional field(s) {list(wrong_fields)} are not "
+                    f"related to model {self.model.__name__}"
                 )
-        self.iterator_attrs = dict((k, v) for k, v in attrs.items() if is_iterator(v))
-        self.model_attrs = dict((k, v) for k, v in attrs.items() if not is_rel_field(k))
-        self.rel_attrs = dict((k, v) for k, v in attrs.items() if is_rel_field(k))
-        self.rel_fields = [
-            x.split("__")[0] for x in self.rel_attrs.keys() if is_rel_field(x)
-        ]
 
-    def _skip_field(self, field: Field) -> bool:
-        from django.contrib.contenttypes.fields import GenericRelation
+        self.iterator_attrs = {
+            k: v for k, v in attrs.items() if isinstance(v, collections.abc.Iterator)
+        }
+        self.model_attrs = {k: v for k, v in attrs.items() if not is_rel_field(k)}
+        self.rel_attrs = {k: v for k, v in attrs.items() if is_rel_field(k)}
+        self.rel_fields = [x.split("__")[0] for x in self.rel_attrs if is_rel_field(x)]
 
+    def _skip_field(self, field: Field) -> bool:  # noqa: C901
         # check for fill optional argument
         if isinstance(self.fill_in_optional, bool):
             field.fill_optional = self.fill_in_optional
@@ -564,7 +594,7 @@ class Baker(Generic[M]):
 
         # Don't Skip related _id fields defined in the iterator attributes
         if (
-            (isinstance(field, OneToOneField) or isinstance(field, ForeignKey))
+            isinstance(field, (OneToOneField, ForeignKey))
             and hasattr(field, "attname")
             and field.attname in self.iterator_attrs
         ):
@@ -574,10 +604,18 @@ class Baker(Generic[M]):
         if isinstance(field, OneToOneField) and self._remote_field(field).parent_link:
             return True
 
-        if isinstance(field, (AutoField, GenericRelation, OrderWrt)):
+        other_fields_to_skip = [
+            AutoField,
+            OrderWrt,
+        ]
+
+        if BAKER_CONTENTTYPES:
+            other_fields_to_skip.extend([GenericRelation, GenericForeignKey])
+
+        if isinstance(field, tuple(other_fields_to_skip)):
             return True
 
-        if all(
+        if all(  # noqa: SIM102
             [
                 field.name not in self.model_attrs,
                 field.name not in self.rel_fields,
@@ -593,29 +631,36 @@ class Baker(Generic[M]):
             ):
                 return True
 
-        if field.name not in self.model_attrs:
+        if field.name not in self.model_attrs:  # noqa: SIM102
             if field.name not in self.rel_fields and (
-                field.null and not field.fill_optional
+                not field.fill_optional and field.null
             ):
                 return True
 
         return False
 
+    def _handle_auto_now(self, instance: Model, attrs: Dict[str, Any]):
+        if not attrs:
+            return
+
+        instance.__class__.objects.filter(pk=instance.pk).update(**attrs)
+
     def _handle_one_to_many(self, instance: Model, attrs: Dict[str, Any]):
         for key, values in attrs.items():
             manager = getattr(instance, key)
 
+            if callable(values):
+                values = values()
+
             for value in values:
                 # Django will handle any operation to persist nested non-persisted FK because
                 # save doesn't do so and, thus, raises constraint errors. That's why save()
                 # only gets called if the object doesn't have a pk and also doesn't hold fk
                 # pointers.
                 fks = any(
-                    [
-                        fk
-                        for fk in value._meta.fields
-                        if isinstance(fk, ForeignKey) or isinstance(fk, OneToOneField)
-                    ]
+                    fk
+                    for fk in value._meta.fields
+                    if isinstance(fk, (ForeignKey, OneToOneField))
                 )
                 if not value.pk and not fks:
                     value.save()
@@ -628,6 +673,9 @@ class Baker(Generic[M]):
 
     def _handle_m2m(self, instance: Model):
         for key, values in self.m2m_dict.items():
+            if callable(values):
+                values = values()
+
             for value in values:
                 if not value.pk:
                     value.save()
@@ -645,16 +693,42 @@ class Baker(Generic[M]):
                     }
                     make(through_model, _using=self._using, **base_kwargs)
 
+    def _handle_generic_foreign_keys(self, instance: Model, attrs: Dict[str, Any]):
+        """Set content type and object id for GenericForeignKey fields."""
+        for field_name, data in attrs.items():
+            ct_field_name = data["content_type_field"]
+            oid_field_name = data["object_id_field"]
+            value = data["value"]
+            if is_iterator(value):
+                value = next(value)
+            if value is None:
+                # when GFK is None, we should try to set the content type and object id to None
+                content_type_field = instance._meta.get_field(ct_field_name)
+                object_id_field = instance._meta.get_field(oid_field_name)
+                if content_type_field.null:
+                    setattr(instance, ct_field_name, None)
+                if object_id_field.null:
+                    setattr(instance, oid_field_name, None)
+            else:
+                setattr(instance, field_name, value)
+                setattr(
+                    instance,
+                    ct_field_name,
+                    contenttypes_models.ContentType.objects.get_for_model(value),
+                )
+                setattr(instance, oid_field_name, value.pk)
+
     def _remote_field(
         self, field: Union[ForeignKey, OneToOneField]
     ) -> Union[OneToOneRel, ManyToOneRel]:
         return field.remote_field
 
-    def generate_value(self, field: Field, commit: bool = True) -> Any:
+    def generate_value(self, field: Field, commit: bool = True) -> Any:  # noqa: C901
         """Call the associated generator with a field passing all required args.
 
         Generator Resolution Precedence Order:
         -- `field.default` - model field default value, unless explicitly overwritten during baking
+        -- `field.db_default` - model field db default value, unless explicitly overwritten
         -- `attr_mapping` - mapping per attribute name
         -- `choices` -- mapping from available field choices
         -- `type_mapping` - mapping from user defined type associated generators
@@ -664,20 +738,28 @@ class Baker(Generic[M]):
         `attr_mapping` and `type_mapping` can be defined easily overwriting the
         model.
         """
-        is_content_type_fk = isinstance(field, ForeignKey) and issubclass(
-            self._remote_field(field).model, contenttypes.models.ContentType
-        )
+        is_content_type_fk = False
+        is_generic_fk = False
+        if BAKER_CONTENTTYPES:
+            is_content_type_fk = isinstance(field, ForeignKey) and issubclass(
+                self._remote_field(field).model, contenttypes_models.ContentType
+            )
+            is_generic_fk = isinstance(field, GenericForeignKey)
+        if is_generic_fk:
+            generator = self.type_mapping[GenericForeignKey]
         # we only use default unless the field is overwritten in `self.rel_fields`
-        if field.has_default() and field.name not in self.rel_fields:
+        elif field.has_default() and field.name not in self.rel_fields:
             if callable(field.default):
                 return field.default()
             return field.default
+        elif getattr(field, "db_default", NOT_PROVIDED) != NOT_PROVIDED:
+            return field.db_default
         elif field.name in self.attr_mapping:
             generator = self.attr_mapping[field.name]
-        elif getattr(field, "choices"):
+        elif field.choices:
             generator = random_gen.gen_from_choices(field.choices)
         elif is_content_type_fk:
-            generator = self.type_mapping[contenttypes.models.ContentType]
+            generator = self.type_mapping[contenttypes_models.ContentType]
         elif generators.get(field.__class__):
             generator = generators.get(field.__class__)
         elif field.__class__ in self.type_mapping:
@@ -695,6 +777,13 @@ class Baker(Generic[M]):
         if field.name in self.rel_fields:
             generator_attrs.update(filter_rel_attrs(field.name, **self.rel_attrs))
 
+        if (
+            field.__class__ in (ForeignKey, OneToOneField, ManyToManyField)
+            and not is_content_type_fk
+        ):
+            # create files also on related models if required
+            generator_attrs["_create_files"] = self.create_files
+
         if not commit:
             generator = getattr(generator, "prepare", generator)
 
@@ -710,33 +799,29 @@ def get_required_values(
     required value is a string, simply fetch the value from the field
     and return.
     """
-    # FIXME: avoid abbreviations
-    rt = {}  # type: Dict[str, Any]
+    required_values = {}  # type: Dict[str, Any]
     if hasattr(generator, "required"):
         for item in generator.required:  # type: ignore[attr-defined]
-
             if callable(item):  # baker can deal with the nasty hacking too!
                 key, value = item(field)
-                rt[key] = value
+                required_values[key] = value
 
             elif isinstance(item, str):
-                rt[item] = getattr(field, item)
+                required_values[item] = getattr(field, item)
 
             else:
                 raise ValueError(
-                    "Required value '%s' is of wrong type. \
-                                  Don't make baker sad."
-                    % str(item)
+                    f"Required value '{item}' is of wrong type. Don't make baker sad."
                 )
 
-    return rt
+    return required_values
 
 
 def filter_rel_attrs(field_name: str, **rel_attrs) -> Dict[str, Any]:
     clean_dict = {}
 
     for k, v in rel_attrs.items():
-        if k.startswith(field_name + "__"):
+        if k.startswith(f"{field_name}__"):
             splitted_key = k.split("__")
             key = "__".join(splitted_key[1:])
             clean_dict[key] = v
@@ -746,6 +831,28 @@ def filter_rel_attrs(field_name: str, **rel_attrs) -> Dict[str, Any]:
     return clean_dict
 
 
+def _save_related_objs(model, objects, _using=None) -> None:
+    """Recursively save all related foreign keys for each entry."""
+    _save_kwargs = {"using": _using} if _using else {}
+
+    fk_fields = [
+        f for f in model._meta.fields if isinstance(f, (OneToOneField, ForeignKey))
+    ]
+
+    for fk in fk_fields:
+        fk_objects = []
+        for obj in objects:
+            fk_obj = getattr(obj, fk.name, None)
+            if fk_obj and not fk_obj.pk:
+                fk_objects.append(fk_obj)
+
+        if fk_objects:
+            _save_related_objs(fk.related_model, fk_objects)
+            for i, fk_obj in enumerate(fk_objects):
+                fk_obj.save(**_save_kwargs)
+                setattr(objects[i], fk.name, fk_obj)
+
+
 def bulk_create(baker: Baker[M], quantity: int, **kwargs) -> List[M]:
     """
     Bulk create entries and all related FKs as well.
@@ -753,43 +860,53 @@ def bulk_create(baker: Baker[M], quantity: int, **kwargs) -> List[M]:
     Important: there's no way to avoid save calls since Django does
     not return the created objects after a bulk_insert call.
     """
-    _save_kwargs = {}
-    if baker._using:
-        _save_kwargs = {"using": baker._using}
-
-    def _save_related_objs(model, objects) -> None:
-        fk_fields = [
-            f
-            for f in model._meta.fields
-            if isinstance(f, OneToOneField) or isinstance(f, ForeignKey)
-        ]
-
-        for fk in fk_fields:
-            fk_objects = []
-            for obj in objects:
-                fk_obj = getattr(obj, fk.name, None)
-                if fk_obj and not fk_obj.pk:
-                    fk_objects.append(fk_obj)
-
-            if fk_objects:
-                _save_related_objs(fk.related_model, fk_objects)
-                for i, fk_obj in enumerate(fk_objects):
-                    fk_obj.save(**_save_kwargs)
-                    setattr(objects[i], fk.name, fk_obj)
-
+    # Create a list of entries by calling the prepare method of the Baker instance
+    # quantity number of times, passing in the additional keyword arguments
     entries = [
         baker.prepare(
             **kwargs,
         )
         for _ in range(quantity)
     ]
-    _save_related_objs(baker.model, entries)
 
+    _save_related_objs(baker.model, entries, _using=baker._using)
+
+    # Use the desired database to create the entries
     if baker._using:
-        # Try to use the desired DB and let Django fail if spanning
-        # relationships without the proper router setup
         manager = baker.model._base_manager.using(baker._using)
     else:
         manager = baker.model._base_manager
 
-    return manager.bulk_create(entries)
+    created_entries = manager.bulk_create(entries)
+
+    # set many-to-many relations from kwargs
+    for entry in created_entries:
+        for field in baker.model._meta.many_to_many:
+            if field.name in kwargs:
+                through_model = getattr(entry, field.name).through
+                through_model.objects.bulk_create(
+                    [
+                        through_model(
+                            **{
+                                field.remote_field.name: entry,
+                                field.related_model._meta.model_name: obj,
+                            }
+                        )
+                        for obj in kwargs[field.name]
+                    ]
+                )
+
+        # set many-to-many relations that are specified using related name from kwargs
+        for field in baker.model._meta.get_fields():
+            if field.many_to_many and hasattr(field, "related_model"):
+                reverse_relation_name = (
+                    field.related_query_name
+                    or field.related_name
+                    or f"{field.related_model._meta.model_name}_set"
+                )
+                if reverse_relation_name in kwargs:
+                    getattr(entry, reverse_relation_name).set(
+                        kwargs[reverse_relation_name]
+                    )
+
+    return created_entries
