@@ -3,15 +3,16 @@ import itertools
 from decimal import Decimal
 from unittest.mock import patch
 
-import pytest
+from django.apps import apps
 from django.conf import settings
-from django.contrib.contenttypes.models import ContentType
 from django.db.models import Manager
 from django.db.models.signals import m2m_changed
 from django.test import TestCase, override_settings
 
+import pytest
+
 from model_bakery import baker, random_gen
-from model_bakery.baker import MAX_MANY_QUANTITY
+from model_bakery.baker import BAKER_CONTENTTYPES, MAX_MANY_QUANTITY
 from model_bakery.exceptions import (
     AmbiguousModelName,
     InvalidQuantityException,
@@ -27,7 +28,7 @@ def test_import_seq_from_baker():
     try:
         from model_bakery.baker import seq  # NoQA
     except ImportError:
-        pytest.fail("{} raised".format(ImportError.__name__))
+        pytest.fail(f"{ImportError.__name__} raised")
 
 
 class TestsModelFinder:
@@ -97,7 +98,7 @@ class TestsBakerCreatesSimpleModel:
             try:
                 baker.make(models.ModelWithImpostorField)
             except TypeError:
-                assert False, "TypeError raised"
+                raise AssertionError("TypeError raised")
 
     def test_make_should_create_one_object(self):
         person = baker.make(models.Person)
@@ -119,14 +120,14 @@ class TestsBakerCreatesSimpleModel:
             models.NonAbstractPerson, name="bob", enjoy_jards_macale=False
         )
         assert isinstance(person, models.NonAbstractPerson)
-        assert "bob" == person.name
+        assert person.name == "bob"
         assert person.enjoy_jards_macale is False
 
     def test_abstract_model_subclass_creation(self):
         instance = baker.make(models.SubclassOfAbstract)
         assert isinstance(instance, models.SubclassOfAbstract)
         assert isinstance(instance, models.AbstractModel)
-        assert isinstance(instance.name, type(""))
+        assert isinstance(instance.name, str)
         assert len(instance.name) == 30
         assert isinstance(instance.height, int)
 
@@ -151,17 +152,18 @@ class TestsBakerRepeatedCreatesSimpleModel(TestCase):
             assert all(p.name == "George Washington" for p in people)
 
     def test_make_quantity_respecting_bulk_create_parameter(self):
-        with self.assertNumQueries(1):
+        query_count = 1
+        with self.assertNumQueries(query_count):
             baker.make(models.Person, _quantity=5, _bulk_create=True)
         assert models.Person.objects.count() == 5
 
-        with self.assertNumQueries(1):
+        with self.assertNumQueries(query_count):
             people = baker.make(
                 models.Person, name="George Washington", _quantity=5, _bulk_create=True
             )
             assert all(p.name == "George Washington" for p in people)
 
-        with self.assertNumQueries(1):
+        with self.assertNumQueries(query_count):
             baker.make(models.NonStandardManager, _quantity=3, _bulk_create=True)
             assert getattr(models.NonStandardManager, "objects", None) is None
             assert (
@@ -207,16 +209,16 @@ class TestsBakerRepeatedCreatesSimpleModel(TestCase):
         )
         assert models.Person.objects.count() == 5
         p1, p2, p3, p4, p5 = models.Person.objects.all().order_by("pk")
-        assert "a" == p1.name
-        assert "d1" == p1.id_document
-        assert "b" == p2.name
-        assert "d2" == p2.id_document
-        assert "c" == p3.name
-        assert "d3" == p3.id_document
-        assert "a" == p4.name
-        assert "d4" == p4.id_document
-        assert "b" == p5.name
-        assert "d5" == p5.id_document
+        assert p1.name == "a"
+        assert p1.id_document == "d1"
+        assert p2.name == "b"
+        assert p2.id_document == "d2"
+        assert p3.name == "c"
+        assert p3.id_document == "d3"
+        assert p4.name == "a"
+        assert p4.id_document == "d4"
+        assert p5.name == "b"
+        assert p5.id_document == "d5"
 
     def test_accepts_generators_with_quantity_for_unique_fields(self):
         baker.make(
@@ -226,10 +228,15 @@ class TestsBakerRepeatedCreatesSimpleModel(TestCase):
         )
         assert models.DummyUniqueIntegerFieldModel.objects.count() == 3
         num_1, num_2, num_3 = models.DummyUniqueIntegerFieldModel.objects.all()
-        assert 1 == num_1.value
-        assert 2 == num_2.value
-        assert 3 == num_3.value
+        assert num_1.value == 1
+        assert num_2.value == 2
+        assert num_3.value == 3
 
+    # skip if auth app is not installed
+    @pytest.mark.skipif(
+        not apps.is_installed("django.contrib.auth"),
+        reason="Django auth app is not installed",
+    )
     def test_generators_work_with_user_model(self):
         from django.contrib.auth import get_user_model
 
@@ -237,9 +244,9 @@ class TestsBakerRepeatedCreatesSimpleModel(TestCase):
         baker.make(User, username=itertools.cycle(["a", "b", "c"]), _quantity=3)
         assert User.objects.count() == 3
         u1, u2, u3 = User.objects.all()
-        assert "a" == u1.username
-        assert "b" == u2.username
-        assert "c" == u3.username
+        assert u1.username == "a"
+        assert u2.username == "b"
+        assert u3.username == "c"
 
 
 @pytest.mark.django_db
@@ -322,7 +329,7 @@ class TestBakerCreatesAssociatedModels(TestCase):
         try:
             baker.make(models.Person, classroom_set=[baker.make(models.Classroom)])
         except TypeError:
-            assert False, "type error raised"
+            raise AssertionError("type error raised")
 
     def test_save_object_instances_when_handling_one_to_many_relations(self):
         owner = baker.make(models.Person)
@@ -332,14 +339,14 @@ class TestBakerCreatesAssociatedModels(TestCase):
             _quantity=2,
         )
 
-        assert 0 == models.Dog.objects.count()  # ensure there're no dogs in our db
+        assert models.Dog.objects.count() == 0  # ensure there are no dogs in our db
         home = baker.make(
             models.Home,
             owner=owner,
             dogs=dogs_set,
         )
         assert home.dogs.count() == 2
-        assert 2 == models.Dog.objects.count()  # dogs in dogs_set were created
+        assert models.Dog.objects.count() == 2  # dogs in dogs_set were created
 
     def test_prepare_fk(self):
         dog = baker.prepare(models.Dog)
@@ -362,16 +369,17 @@ class TestBakerCreatesAssociatedModels(TestCase):
         assert models.Person.objects.all().count() == 5
 
     def test_bulk_create_multiple_one_to_one(self):
-        with self.assertNumQueries(6):
+        query_count = 6
+        with self.assertNumQueries(query_count):
             baker.make(models.LonelyPerson, _quantity=5, _bulk_create=True)
 
         assert models.LonelyPerson.objects.all().count() == 5
         assert models.Person.objects.all().count() == 5
 
     def test_chaining_bulk_create_reduces_query_count(self):
-        qtd = 5
-        with self.assertNumQueries(3):
-            baker.make(models.Person, _quantity=qtd, _bulk_create=True)
+        query_count = 3
+        with self.assertNumQueries(query_count):
+            baker.make(models.Person, _quantity=5, _bulk_create=True)
             person_iter = models.Person.objects.all().iterator()
             baker.make(
                 models.LonelyPerson,
@@ -385,7 +393,8 @@ class TestBakerCreatesAssociatedModels(TestCase):
         assert models.Person.objects.all().count() == 5
 
     def test_bulk_create_multiple_fk(self):
-        with self.assertNumQueries(6):
+        query_count = 6
+        with self.assertNumQueries(query_count):
             baker.make(models.PaymentBill, _quantity=5, _bulk_create=True)
 
         assert models.PaymentBill.objects.all().count() == 5
@@ -396,7 +405,7 @@ class TestBakerCreatesAssociatedModels(TestCase):
         assert store.employees.count() == 5
         assert store.customers.count() == 5
 
-    def test_regresstion_many_to_many_field_is_accepted_as_kwargs(self):
+    def test_regression_many_to_many_field_is_accepted_as_kwargs(self):
         employees = baker.make(models.Person, _quantity=3)
         customers = baker.make(models.Person, _quantity=3)
 
@@ -471,7 +480,7 @@ class TestBakerCreatesAssociatedModels(TestCase):
 
     def test_nullable_many_to_many_is_not_created_even_if_flagged(self):
         classroom = baker.make(models.Classroom, make_m2m=True)
-        assert not classroom.students.count()
+        assert classroom.students.count() == 0
 
     def test_m2m_changed_signal_is_fired(self):
         # TODO: Use object attrs instead of mocks for Django 1.4 compat
@@ -490,7 +499,7 @@ class TestBakerCreatesAssociatedModels(TestCase):
         assert kid.enjoy_jards_macale is True
         assert kid.name == "Mike"
 
-    def test_creating_person_from_factory_using_paramters(self):
+    def test_creating_person_from_factory_using_parameters(self):
         person_baker_ = baker.Baker(models.Person)
         person = person_baker_.make(
             enjoy_jards_macale=False, age=20, gender="M", name="John"
@@ -502,17 +511,17 @@ class TestBakerCreatesAssociatedModels(TestCase):
 
     def test_ForeignKey_model_field_population(self):
         dog = baker.make(models.Dog, breed="X1", owner__name="Bob")
-        assert "X1" == dog.breed
-        assert "Bob" == dog.owner.name
+        assert dog.breed == "X1"
+        assert dog.owner.name == "Bob"
 
     def test_ForeignKey_model_field_population_should_work_with_prepare(self):
         dog = baker.prepare(models.Dog, breed="X1", owner__name="Bob")
-        assert "X1" == dog.breed
-        assert "Bob" == dog.owner.name
+        assert dog.breed == "X1"
+        assert dog.owner.name == "Bob"
 
     def test_ForeignKey_model_field_population_for_not_required_fk(self):
         user = baker.make(models.User, profile__email="a@b.com")
-        assert "a@b.com" == user.profile.email
+        assert user.profile.email == "a@b.com"
 
     def test_does_not_creates_null_ForeignKey(self):
         user = baker.make(models.User)
@@ -524,26 +533,26 @@ class TestBakerCreatesAssociatedModels(TestCase):
 
     def test_ensure_recursive_ForeignKey_population(self):
         bill = baker.make(models.PaymentBill, user__profile__email="a@b.com")
-        assert "a@b.com" == bill.user.profile.email
+        assert bill.user.profile.email == "a@b.com"
 
     def test_field_lookup_for_m2m_relationship(self):
         store = baker.make(models.Store, suppliers__gender="M")
         suppliers = store.suppliers.all()
         assert suppliers
         for supplier in suppliers:
-            assert "M" == supplier.gender
+            assert supplier.gender == "M"
 
     def test_field_lookup_for_one_to_one_relationship(self):
         lonely_person = baker.make(models.LonelyPerson, only_friend__name="Bob")
-        assert "Bob" == lonely_person.only_friend.name
+        assert lonely_person.only_friend.name == "Bob"
 
-    def test_allow_create_fkey_related_model(self):
+    def test_allow_create_fk_related_model(self):
         try:
             person = baker.make(
                 models.Person, dog_set=[baker.make(models.Dog), baker.make(models.Dog)]
             )
         except TypeError:
-            assert False, "type error raised"
+            raise AssertionError("type error raised")
 
         assert person.dog_set.count() == 2
 
@@ -556,9 +565,9 @@ class TestBakerCreatesAssociatedModels(TestCase):
 
         assert person.pk
         assert person.one_related.pk
-        assert 1, person.fk_related.count()
-        assert "Foo" == person.one_related.name
-        assert "Bar" == person.fk_related.get().name
+        assert person.fk_related.count() == 1
+        assert person.one_related.name == "Foo"
+        assert person.fk_related.get().name == "Bar"
 
     def test_field_lookup_for_related_field_does_not_work_with_prepare(self):
         person = baker.prepare(
@@ -568,7 +577,7 @@ class TestBakerCreatesAssociatedModels(TestCase):
         )
 
         assert not person.pk
-        assert 0 == models.RelatedNamesModel.objects.count()
+        assert models.RelatedNamesModel.objects.count() == 0
 
     def test_ensure_reverse_fk_for_many_to_one_is_working(self):
         """This is a regression test to make sure issue 291 is fixed."""
@@ -592,12 +601,15 @@ class TestHandlingUnsupportedModels:
     def test_unsupported_model_raises_an_explanatory_exception(self):
         try:
             baker.make(models.UnsupportedModel)
-            assert False, "Should have raised a TypeError"
+            raise AssertionError("Should have raised a TypeError")
         except TypeError as e:
             assert "not supported" in repr(e)
             assert "field unsupported_field" in repr(e)
 
 
+@pytest.mark.skipif(
+    not BAKER_CONTENTTYPES, reason="Django contenttypes framework is not installed"
+)
 @pytest.mark.django_db
 class TestHandlingModelsWithGenericRelationFields:
     def test_create_model_with_generic_relation(self):
@@ -605,18 +617,28 @@ class TestHandlingModelsWithGenericRelationFields:
         assert isinstance(dummy, models.DummyGenericRelationModel)
 
 
+@pytest.mark.skipif(
+    not BAKER_CONTENTTYPES, reason="Django contenttypes framework is not installed"
+)
 @pytest.mark.django_db
 class TestHandlingContentTypeField:
     def test_create_model_with_contenttype_field(self):
+        from django.contrib.contenttypes.models import ContentType
+
         dummy = baker.make(models.DummyGenericForeignKeyModel)
         assert isinstance(dummy, models.DummyGenericForeignKeyModel)
         assert isinstance(dummy.content_type, ContentType)
 
 
+@pytest.mark.skipif(
+    not BAKER_CONTENTTYPES, reason="Django contenttypes framework is not installed"
+)
 class TestHandlingContentTypeFieldNoQueries:
     def test_create_model_with_contenttype_field(self):
+        from django.contrib.contenttypes.models import ContentType
+
         # Clear ContentType's internal cache so that it *will* try to connect to
-        # the database in order to fetch the corresponding ContentType model for
+        # the database to fetch the corresponding ContentType model for
         # a randomly chosen model.
         ContentType.objects.clear_cache()
 
@@ -687,7 +709,7 @@ class TestFillBlanksTestCase:
         )
         assert len(dummy.blank_char_field) == 50
 
-    def test_fill_field_optinal_when_preparing(self):
+    def test_fill_field_optional_when_preparing(self):
         dummy = baker.prepare(
             models.DummyBlankFieldsModel, _fill_optional=["blank_char_field"]
         )
@@ -783,7 +805,7 @@ class TestBakerHandlesModelWithNext:
         assert instance.id
         assert instance.fk.id
         assert instance.fk.attr
-        assert "foo" == instance.fk.next()
+        assert instance.fk.next() == "foo"
 
 
 @pytest.mark.django_db
@@ -792,11 +814,11 @@ class TestBakerHandlesModelWithList:
         instance = baker.make(models.BaseModelForList, fk=["foo"])
 
         assert instance.id
-        assert ["foo"] == instance.fk
+        assert instance.fk == ["foo"]
 
 
 @pytest.mark.django_db
-class TestBakerGeneratesIPAdresses:
+class TestBakerGeneratesIPAddresses:
     def test_create_model_with_valid_ips(self):
         form_data = {
             "ipv4_field": random_gen.gen_ipv4(),
@@ -811,11 +833,11 @@ class TestBakerAllowsSaveParameters(TestCase):
 
     def test_allows_save_kwargs_on_baker_make(self):
         owner = baker.make(models.Person)
-        dog = baker.make(models.ModelWithOverridedSave, _save_kwargs={"owner": owner})
+        dog = baker.make(models.ModelWithOverwrittenSave, _save_kwargs={"owner": owner})
         assert owner == dog.owner
 
         dog1, dog2 = baker.make(
-            models.ModelWithOverridedSave, _save_kwargs={"owner": owner}, _quantity=2
+            models.ModelWithOverwrittenSave, _save_kwargs={"owner": owner}, _quantity=2
         )
         assert dog1.owner == owner
         assert dog2.owner == owner
@@ -925,7 +947,7 @@ class TestBakerSupportsMultiDatabase(TestCase):
         dog_qs = models.Dog.objects.using(settings.EXTRA_DB).all()
         person_qs = models.Person.objects.using(settings.EXTRA_DB).all()
         assert dog_qs.count() == 5
-        # since we're using recipes, all dogs belongs to the same owner
+        # since we're using recipes, all dogs belong to the same owner
         assert person_qs.count() == 1
         for dog in dogs:
             dog.refresh_from_db()
@@ -1032,3 +1054,73 @@ class TestBakerMakeCanFetchInstanceFromDefaultManager:
             _from_manager="objects",
         )
         assert movie.title == movie.name
+
+
+@pytest.mark.django_db
+class TestCreateM2MWhenBulkCreate(TestCase):
+    def test_create(self):
+        person = baker.make(models.Person)
+
+        with self.assertNumQueries(11):
+            baker.make(
+                models.Classroom, students=[person], _quantity=10, _bulk_create=True
+            )
+        c1, c2 = models.Classroom.objects.all()[:2]
+        assert list(c1.students.all()) == list(c2.students.all()) == [person]
+
+    def test_make_should_create_objects_using_reverse_name(self):
+        classroom = baker.make(models.Classroom)
+
+        with self.assertNumQueries(21):
+            baker.make(
+                models.Person,
+                classroom_set=[classroom],
+                _quantity=10,
+                _bulk_create=True,
+            )
+        s1, s2 = models.Person.objects.all()[:2]
+        assert (
+            list(s1.classroom_set.all()) == list(s2.classroom_set.all()) == [classroom]
+        )
+
+
+class TestBakerSeeded:
+    @pytest.fixture()
+    def reset_seed(self):
+        old_state = random_gen.baker_random.getstate()
+        yield
+        random_gen.baker_random.setstate(old_state)
+        baker.Baker._global_seed = baker.Baker.SENTINEL
+
+    @pytest.mark.django_db
+    def test_seed(self, reset_seed):
+        baker.seed(1)
+        assert baker.Baker._global_seed == 1
+        assert random_gen.gen_integer() == 55195912693
+
+    @pytest.mark.django_db
+    def test_unseeded(self):
+        assert baker.Baker._global_seed is baker.Baker.SENTINEL
+
+
+class TestAutoNowFields:
+    @pytest.mark.django_db
+    @pytest.mark.parametrize("use_tz", [False, True])
+    def test_make_with_auto_now(self, use_tz, settings):
+        settings.USE_TZ = use_tz
+        tzinfo = datetime.timezone.utc if use_tz else None
+
+        now = datetime.datetime(2023, 10, 20, 15, 30).replace(tzinfo=tzinfo)
+
+        instance = baker.make(
+            models.ModelWithAutoNowFields,
+            created=now,
+            updated=now,
+            sent_date=now,
+        )
+
+        instance.refresh_from_db()
+
+        assert instance.created == now
+        assert instance.updated == now
+        assert instance.sent_date == now
