@@ -4,12 +4,13 @@ from decimal import Decimal
 from random import choice  # noqa
 from unittest.mock import patch
 
-import pytest
 from django.utils.timezone import now
 
+import pytest
+
 from model_bakery import baker
 from model_bakery.exceptions import InvalidQuantityException, RecipeIteratorEmpty
-from model_bakery.recipe import Recipe, RecipeForeignKey, foreign_key
+from model_bakery.recipe import Recipe, RecipeForeignKey, foreign_key, seq
 from model_bakery.timezone import tz_aware
 from tests.generic.baker_recipes import SmallDogRecipe, pug
 from tests.generic.models import (
@@ -44,7 +45,7 @@ def test_import_seq_from_recipe():
     try:
         from model_bakery.recipe import seq  # NoQA
     except ImportError:
-        pytest.fail("{} raised".format(ImportError.__name__))
+        pytest.fail(f"{ImportError.__name__} raised")
 
 
 def test_import_recipes():
@@ -140,7 +141,7 @@ class TestDefiningRecipes:
     def test_make_recipe_without_all_model_needed_data(self):
         person_recipe = Recipe(Person, name="John Doe")
         person = person_recipe.make()
-        assert "John Doe" == person.name
+        assert person.name == "John Doe"
         assert person.nickname
         assert person.age
         assert person.bio
@@ -153,7 +154,7 @@ class TestDefiningRecipes:
     def test_prepare_recipe_without_all_model_needed_data(self):
         person_recipe = Recipe(Person, name="John Doe")
         person = person_recipe.prepare()
-        assert "John Doe" == person.name
+        assert person.name == "John Doe"
         assert person.nickname
         assert person.age
         assert person.bio
@@ -164,13 +165,11 @@ class TestDefiningRecipes:
         assert not person.id
 
     def test_defining_recipes_str(self):
-        from model_bakery.recipe import seq
-
         p = Recipe("generic.Person", name=seq("foo"))
         try:
             p.make(_quantity=5)
         except AttributeError as e:
-            pytest.fail("%s" % e)
+            pytest.fail(f"{e}")
 
 
 @pytest.mark.django_db
@@ -236,7 +235,7 @@ class TestExecutingRecipes:
     def test_make_extended_recipe(self):
         extended_dog = baker.make_recipe("tests.generic.extended_dog")
         assert extended_dog.breed == "Super basset"
-        # No side effects happened due to extension
+        # No side effects happened due to a recipe extension
         base_dog = baker.make_recipe("tests.generic.dog")
         assert base_dog.breed == "Pug"
 
@@ -252,7 +251,7 @@ class TestExecutingRecipes:
         assert not dog.id
         assert dog.owner.id
 
-    def test_make_recipe_with_quantity_parameter_respection_model_args(self):
+    def test_make_recipe_with_quantity_parameter_respecting_model_args(self):
         people = baker.make_recipe(
             "tests.generic.person", _quantity=3, name="Dennis Ritchie", age=70
         )
@@ -285,7 +284,7 @@ class TestExecutingRecipes:
             assert isinstance(person, Person)
             assert person.id is None
 
-    def test_prepare_recipe_with_quantity_parameter_respection_model_args(self):
+    def test_prepare_recipe_with_quantity_parameter_respecting_model_args(self):
         people = baker.prepare_recipe(
             "tests.generic.person", _quantity=3, name="Dennis Ritchie", age=70
         )
@@ -328,7 +327,7 @@ class TestExecutingRecipes:
         owner = baker.make(Person)
 
         dog = baker.make_recipe(
-            "tests.generic.overrided_save", _save_kwargs={"owner": owner}
+            "tests.generic.overwritten_save", _save_kwargs={"owner": owner}
         )
         assert owner == dog.owner
 
@@ -339,10 +338,10 @@ class TestExecutingRecipes:
     def test_ip_fields_with_start(self):
         first, second = baker.make_recipe("tests.generic.ip_fields", _quantity=2)
 
-        assert "127.0.0.2" == first.ipv4_field
-        assert "2001:12f8:0:28::4" == first.ipv6_field
-        assert "127.0.0.4" == second.ipv4_field
-        assert "2001:12f8:0:28::6" == second.ipv6_field
+        assert first.ipv4_field == "127.0.0.2"
+        assert first.ipv6_field == "2001:12f8:0:28::4"
+        assert second.ipv4_field == "127.0.0.4"
+        assert second.ipv6_field == "2001:12f8:0:28::6"
 
 
 @pytest.mark.django_db
@@ -407,6 +406,11 @@ class TestForeignKey:
         assert lady.dog_set.all()[0].breed == "Pug"
         assert lady.dog_set.all()[1].breed == "Basset"
 
+    def test_related_models_recipes_make_mutiple(self):
+        ladies = baker.make_recipe("tests.generic.dog_lady", _quantity=2)
+        assert ladies[0].dog_set.count() == 2
+        assert ladies[1].dog_set.count() == 2
+
     def test_nullable_related(self):
         nullable = baker.make_recipe("tests.generic.nullable_related")
         assert nullable.dummynullfieldsmodel_set.count() == 1
@@ -417,7 +421,7 @@ class TestForeignKey:
 
     def test_one_to_one_relationship(self):
         lonely_people = baker.make_recipe("tests.generic.lonely_person", _quantity=2)
-        friend_ids = set([x.only_friend.id for x in lonely_people])
+        friend_ids = {x.only_friend.id for x in lonely_people}
         assert len(friend_ids) == 2
 
 
@@ -448,8 +452,6 @@ class TestSequences:
         assert person.name == "joe3"
 
     def test_increment_for_strings_with_suffix(self):
-        from model_bakery.recipe import seq  # NoQA
-
         fred_person = person_recipe.extend(email=seq("fred", suffix="@example.com"))
         person = fred_person.make()
         assert person.email == "fred1@example.com"
@@ -459,8 +461,6 @@ class TestSequences:
         assert person.email == "fred3@example.com"
 
     def test_increment_for_fks(self):
-        from model_bakery.recipe import seq  # NoQA
-
         profiles = baker.make(Profile, _quantity=3)
         start_id = profiles[0].id
         seq_user = user_recipe.extend(username="name", profile_id=seq(start_id))
@@ -470,8 +470,6 @@ class TestSequences:
         assert user.profile_id == start_id + 2
 
     def test_increment_for_one_to_one(self):
-        from model_bakery.recipe import seq  # NoQA
-
         people = baker.make(Person, _quantity=3)
         start_id = people[0].id
         seq_lonely_person = lonely_person_recipe.extend(only_friend_id=seq(start_id))
@@ -481,17 +479,12 @@ class TestSequences:
         assert user.only_friend_id == start_id + 2
 
     def test_increment_for_strings_with_bad_suffix(self):
-        from model_bakery.recipe import seq  # NoQA
-
-        # Bad suffix
         bob_person = person_recipe.extend(email=seq("bob", suffix=42))
         with pytest.raises(TypeError) as exc:
             bob_person.make()
             assert str(exc.value) == "Sequences suffix can only be a string"
 
     def test_increment_for_strings_with_suffix_and_start(self):
-        from model_bakery.recipe import seq  # NoQA
-
         fred_person = person_recipe.extend(
             email=seq("fred", start=5, suffix="@example.com")
         )
@@ -532,8 +525,6 @@ class TestSequences:
         assert dummy.default_float_field == 4.23
 
     def test_increment_for_numbers_with_suffix(self):
-        from model_bakery.recipe import seq  # NoQA
-
         with pytest.raises(TypeError) as exc:
             baker.make_recipe(
                 "tests.generic.serial_numbers",
@@ -551,9 +542,9 @@ class TestSequences:
 
     def test_creates_unique_field_recipe_using_quantity_argument(self):
         dummies = baker.make_recipe("tests.generic.dummy_unique_field", _quantity=3)
-        assert 11 == dummies[0].value
-        assert 12 == dummies[1].value
-        assert 13 == dummies[2].value
+        assert dummies[0].value == 11
+        assert dummies[1].value == 12
+        assert dummies[2].value == 13
 
     def test_increment_by_3(self):
         dummy = baker.make_recipe("tests.generic.serial_numbers_by")
@@ -614,20 +605,20 @@ class TestSequences:
 class TestIterators:
     def test_accepts_generators(self):
         r = Recipe(DummyBlankFieldsModel, blank_char_field=itertools.cycle(["a", "b"]))
-        assert "a" == r.make().blank_char_field
-        assert "b" == r.make().blank_char_field
-        assert "a" == r.make().blank_char_field
+        assert r.make().blank_char_field == "a"
+        assert r.make().blank_char_field == "b"
+        assert r.make().blank_char_field == "a"
 
     def test_accepts_iterators(self):
         r = Recipe(DummyBlankFieldsModel, blank_char_field=iter(["a", "b", "c"]))
-        assert "a" == r.make().blank_char_field
-        assert "b" == r.make().blank_char_field
-        assert "c" == r.make().blank_char_field
+        assert r.make().blank_char_field == "a"
+        assert r.make().blank_char_field == "b"
+        assert r.make().blank_char_field == "c"
 
     def test_empty_iterator_exception(self):
         r = Recipe(DummyBlankFieldsModel, blank_char_field=iter(["a", "b"]))
-        assert "a" == r.make().blank_char_field
-        assert "b" == r.make().blank_char_field
+        assert r.make().blank_char_field == "a"
+        assert r.make().blank_char_field == "b"
         with pytest.raises(RecipeIteratorEmpty):
             r.make()
 
